################################
# MDP Microbes Application Study
#
# This script analyzes results from the MDP estimations and 
# the IRT estimates from the posttest and the MDP level results
# Code written by Michelle LaMar


#######################################################################
# Read in the MDP estimation results

modelNames = c("mod1", "mod2", "mod2p")

# Get table of model parameters:
modTableTmp = NULL
for(m in modelNames) {
	modFile = paste("mic_modEst_", m, ".out", sep = "")
	mTab = read.table(modFile, stringsAsFactors = F)
	colnames(mTab) = c("params", m)
	if(is.null(modTableTmp)) 
		modTableTmp = mTab
	else
		modTableTmp = merge(modTableTmp, mTab, all = T, sort = F)
	
}

modTabCln = t(modTableTmp [,-1])
colnames(modTabCln) = modTableTmp[,1]
modTabCln= as.data.frame(modTabCln)
modTabCln = modTabCln[,-c(3:10,13,14,16:18)]
modTabCln$log_likelihood = as.numeric(as.character(modTabCln$log_likelihood))

modTabCln$nParams = c(2,4,4)

modTabCln$deviance = -2 * modTabCln$log_likelihood
modTabCln$AIC = 2 * modTabCln$nParams + modTabCln$deviance

############################
# Table 9 in the Paper
# 
t(modTabCln)


# Now read in the person ability estimates:

abilDatMleList = vector(mode = "list")
logThetaMDPEsts = NULL
thetaMDPEsts = NULL

for(m in modelNames) {
	abilDatMleList[[m]] = read.table(paste("mic_perEst_",m,".mle", sep = ""))
	colnames(abilDatMleList[[m]]) = c("studID", paste("thetaEst_", m,"_mle", sep = ""))
	if(is.null(logThetaMDPEsts)) {
		logThetaMDPEsts = abilDatMleList[[m]]
		thetaMDPEsts = abilDatMleList[[m]]
		thetaMDPEsts[,2] = exp(thetaMDPEsts[,2])
	} else {
		logThetaMDPEsts = merge(logThetaMDPEsts, abilDatMleList[[m]], sort = FALSE, all = TRUE)
		tmpDat = abilDatMleList[[m]]
		tmpDat[,2] = exp(tmpDat[,2])
		thetaMDPEsts = merge(thetaMDPEsts, tmpDat, sort = FALSE, all = TRUE)
	}
}

#############################
# Figure 5 in the paper
hist(logThetaMDPEsts$thetaEst_mod1_mle, xlab = expression(paste("model 1  ", log(hat(beta)) ) ), main = "" ,breaks = 24)


###########################################################################
# Read in posttest estimates and Microbe IRT estimates.
# These models were all estimated with ConQuest software

postEsts_mle = read.table("MicPostAll.mle") # The ConQuest output parameter estimation
postScores = read.table("MicPostAll_cq.txt")  # The ConQuest prepared data.  We read this in only to get the student IDs

postDat = data.frame(studID = postScores$V1, postEst_mle = postEsts_mle$V4)

# Now get Microbe IRT estimates
# These estimates come from ConQuest runs with the two IRT models:

irtMleEsts = read.table("MicIRT.mle")    # This is the "First Try" dichotomous model
irtMlePcEsts = read.table("MicIRT_PC.mle")   # This is the partial credit model

# Original data files, read in for the student IDs:
irtDat = read.table("MicIRT_cq.txt")  
irtPCDat = read.table("MicIRT_PC_cq.txt")

irtDat = data.frame(studID = irtDat$V1, irtFtEst_mle = irtMleEsts$V4, irtPcEst_mle = irtMlePcEsts$V4)

ptCompDat = merge(postDat, thetaMDPEsts)
allCompDat = merge(ptCompDat, irtDat)

corDat = cbind(PT_Cor_pears = cor(allCompDat[,-1])[1,-1], PT_Cor_spear = cor(allCompDat[,-1], method = "sp")[1,-1])

#####################################
# Table 10 in the paper

round(corDat[-2,], 3)  # No need to show Model 2, we've already talked about how it failed

