################################
# Sim Study 2
#
# Looking for ability to distinguish between population-level difference
# in ability and motivation. 
# (Originally called study 's4')
# This code analyzes the estimation results from the simulation study.
# Note that code for generation and estimation can be found in SimStudy2_genEstRecs.R
#
# Code written by Michelle LaMar

# Start in the directory that contains the directories with the generated play data
simDir =  getwd()
setwd(simDir)

############################################
# Utility functions
#

rmse = function(est, true) {
	return(sqrt( mean( (est - true)^2)))
}

#############################################

# Set overall parameters.  These do not change for this sim:
N = 200
nReps = 25

popNames = c("High Ab : High Mot", "High Ab : Low Mot", "Low Ab : High Mot", "Low Ab : Low Mot")
popLabs = c("HAHM", "HALM", "LAHM", "LALM")

# Person Param Files
trueThetaFile_HA = "S4_trueLogTheta_HA.txt"
trueThetaFile_LA = "S4_trueLogTheta_LA.txt"

trueLogTheta_ha = read.table(trueThetaFile_HA)$V2
trueTheta_ha = exp(trueLogTheta_ha)
trueLogTheta_la = read.table(trueThetaFile_LA)$V2
trueTheta_la = exp(trueLogTheta_la)

startReps = 1
endReps = nReps

# Create string versions of the run numbers for the file names
runStr = formatC(1:nReps, width = nchar(as.character(nReps)), flag = "0")

boardList = c( "PlusPlusBoard", "ThinCross", "Cross_1100", "Diamond")

dirList = paste(boardList, "gc1_ec1", sep = "_")
boardFiles = paste("../../boards/", boardList,".txt", sep = "")


S4_dirList = vector()
for(b in 1:length(boardList)) {
for(c in 1:length(popLabs)) { 

	dir = paste(boardList[b],popLabs[c], "est",sep = "_")
	S4_dirList = c(S4_dirList, dir)
}
}

trueLogThetaList = vector(mode = "list")
trueThetaList = vector(mode = "list")

for(b in 1:length(boardList)) {
	ind = (b-1)*4
	trueLogThetaList[[ind+1]]= trueLogTheta_ha
	trueLogThetaList[[ind+2]]= trueLogTheta_ha
	trueLogThetaList[[ind+3]]= trueLogTheta_la
	trueLogThetaList[[ind+4]]= trueLogTheta_la

	trueThetaList[[ind+1]]= trueTheta_ha
	trueThetaList[[ind+2]]= trueTheta_ha
	trueThetaList[[ind+3]]= trueTheta_la
	trueThetaList[[ind+4]]= trueTheta_la

}

# Now the true pop & reward params
trueModParams = matrix(0, nrow = length(popLabs)*length(boardList), ncol = 3)
pop_ha = c(mean(trueLogTheta_ha), sd(trueLogTheta_ha))
pop_la = c(mean(trueLogTheta_la), sd(trueLogTheta_la))
rew_hm = c(-0.05)
rew_lm = c(-0.75)

for(b in 1:length(boardList)) {
	ind = (b-1)*4
	trueModParams[ind+1,]= c(pop_ha, rew_hm)
	trueModParams[ind+2,]= c(pop_ha, rew_lm)
	trueModParams[ind+3,]= c(pop_la, rew_hm)
	trueModParams[ind+4,]= c(pop_la, rew_lm)
}

# Collect up all the data into a couple of data frames
# We will track the theta errors for the normal distribution (log(beta))

boardNames = boardList
studStr = formatC(1:N, width = nchar(as.character(N)), flag = "0")

S4_logThetaEstList = vector(mode = "list")
S4_popEstList = vector(mode = "list")
meanEstLogThetaMat = matrix(NA, ncol = N, nrow = length(S4_dirList))
sdEstLogThetaMat = matrix(NA, ncol = N, nrow = length(S4_dirList))
perStudLogBiasMat = matrix(NA, ncol = N, nrow = length(S4_dirList))
perStudLogRmseMat = matrix(NA, ncol = N, nrow = length(S4_dirList))

for(d in 1:length(S4_dirList)) {
	setwd(simDir)
	setwd(S4_dirList[d])
	# Get the file lists:
	allFiles = list.files()
	thetaEstFiles = grep("S4_thetaEst", allFiles, value = T)
	modEstFiles = grep("S4_modEst", allFiles, value = T)

	S4_logThetaEstList[[d]] = matrix(NA, nrow = length(thetaEstFiles), ncol = N, dimnames = list(paste("run",runStr[1:length(thetaEstFiles)], sep="_"), paste("stud", studStr, sep="_")))
	S4_popEstList[[d]] = matrix(NA, nrow = length(modEstFiles), ncol = 3, 
			dimnames = list(paste("run",runStr[1:length(modEstFiles)], sep="_"), 
			c("thetaMuHat", "thetaSigmaHat", "RmoveHat")))
	
	run = 1
	for(thFile in thetaEstFiles) {
		S4_logThetaEstList[[d]][run,] = read.table(thFile)$V2
		run = run + 1
	}

	run = 1
	for(mFile in modEstFiles) {
		S4_popEstList[[d]][run,] = read.table(mFile)$V2	
		run = run + 1
	}

	meanEstLogThetaMat[d,] = apply(S4_logThetaEstList[[d]], 2, mean)
	sdEstLogThetaMat[d,] = apply(S4_logThetaEstList[[d]], 2, sd)

	perStudLogBiasMat[d,] =  apply(S4_logThetaEstList[[d]], 2, mean) - trueLogThetaList[[d]]

	for(s in 1:N) {
		perStudLogRmseMat[d,s] = rmse(S4_logThetaEstList[[d]][,s], trueLogThetaList[[d]][s])
	}

	setwd("..")
}

rownames(meanEstLogThetaMat) = S4_dirList
rownames(sdEstLogThetaMat) = S4_dirList
rownames(perStudLogBiasMat) = S4_dirList
rownames(perStudLogRmseMat ) = S4_dirList


# Population Parameters:
popMat = matrix(nrow = length(S4_dirList), ncol = 6)

for(b in 1:length(S4_dirList)) {
	popMat[b,c(1,3,5)] = apply(S4_popEstList[[b]], 2, mean)
	popMat[b,c(2,4,6)] = apply(S4_popEstList[[b]], 2, sd)

}
colnames(popMat) = c("Mean mu_theta", "SD mu_theta", "Mean sigma_theta", "SD sigma_theta",
		 "Mean R_move", "SD R_move")
rownames(popMat) = S4_dirList


# Model Parameter Errors:
popErrMat = matrix(nrow = length(S4_dirList), ncol = 6)

for(b in 1:length(S4_dirList)) {
	
	popErrMat[b,1] = mean(S4_popEstList[[b]][,1]) - trueModParams[b,1]
	popErrMat[b,2] = rmse(S4_popEstList[[b]][,1],rep(trueModParams[1],length(S4_popEstList[[b]][,1])))

	popErrMat[b,3] = mean(S4_popEstList[[b]][,2]) - trueModParams[b,2]
	popErrMat[b,4] = rmse(S4_popEstList[[b]][,2],rep(trueModParams[b,2],length(S4_popEstList[[b]][,1])))

	popErrMat[b,5] = mean(S4_popEstList[[b]][,3]) - trueModParams[b,3]
	popErrMat[b,6] = rmse(S4_popEstList[[b]][,3],rep(trueModParams[b,3],length(S4_popEstList[[b]][,1])))

}
colnames(popErrMat) = c("Bias mu_theta", "RMSE mu_theta", "Bias sigma_theta", "RMSE sigma_theta",
	 "Bias R_move", "RMSE R_move")
rownames(popErrMat) = S4_dirList


# Total bias and RMSE for student level theta
logThetaErrMat = matrix(nrow = length(S4_dirList), ncol = 4)

for(b in 1:length(S4_dirList)) {

	perStudLogErr = apply(S4_logThetaEstList[[b]], 1, function(x) { x - trueLogThetaList[[b]]})
	logThetaErrMat[b,1] = mean(perStudLogErr)
	logThetaErrMat[b,2] = sqrt(mean(perStudLogErr^2))
	logThetaErrMat[b,3] = sd(perStudLogErr)/sqrt(length(perStudLogErr))
	logThetaErrMat[b,4] = sqrt(sd(perStudLogErr^2)/sqrt(length(perStudLogErr)))
}
rownames(logThetaErrMat) = S4_dirList
colnames(logThetaErrMat) = c("Bias_theta", "RMSE_theta", "Bias_se", "RMSE_se")


# Create a data frame for population estimate ANOVA:

boardVec = vector()
abilityVec = vector()
motiveVec = vector()
popLabVec = vector()
popEstTmpMat = matrix(ncol = 6)
ind = 1
for(b in 1:length(boardList)) {
for(c in 1:length(popLabs)) { 
	boardVec = c(boardVec, rep(boardList[b], nrow(S4_popEstList[[ind]])))
	abilityVec = c(abilityVec, rep(unlist(strsplit(popLabs[c], ""))[1], nrow(S4_popEstList[[ind]])))
	motiveVec = c(motiveVec, rep(unlist(strsplit(popLabs[c], ""))[3], nrow(S4_popEstList[[ind]])))
	popLabVec = c(popLabVec, rep(popLabs[c], nrow(S4_popEstList[[ind]])))

	errMat = t(apply(S4_popEstList[[ind]], 1, function(x) {x - trueModParams[ind,]}))
	colnames(errMat) = c("muErr", "sigErr", "RmoveErr")
	popEstTmpMat = rbind(popEstTmpMat, cbind(S4_popEstList[[ind]], errMat))
	ind = ind + 1
}
}


popEstTmpMat = popEstTmpMat[-1,]
popParamDat = data.frame(board = boardVec, ability = abilityVec, motive = motiveVec, popLab = popLabVec, popEstTmpMat)

popEsts = aggregate(cbind(thetaMuHat, thetaSigmaHat, RmoveHat) ~ popLab, data = popParamDat, mean)
popSDs = aggregate(cbind(thetaMuHat, thetaSigmaHat, RmoveHat) ~ popLab, data = popParamDat, sd)
popEstTab = data.frame(mu = popEsts[,2], mu_sd = popSDs[,2],
			sig = popEsts[,3], sig_sd = popSDs[,3],
			Rmove = popEsts[,4], Rmove_sd = popSDs[,4])

####################################################
# **** Table 7 (right hand side) for Paper: ****** #
####################################################
round(popEstTab,3)


# Get aggregated error measures:
biasPopMat = aggregate(cbind(muErr, sigErr, RmoveErr) ~ popLab, data = popParamDat, mean)
MSE = aggregate(cbind(muErr^2, sigErr^2, RmoveErr^2) ~ popLab, data = popParamDat, mean)
RMSEPopMat = sqrt(MSE[,2:4])
popCondErrs = data.frame(MuBias = biasPopMat[,2], MuRMSE = RMSEPopMat[,1],
			SigmaBias = biasPopMat[,3], SigmaRMSE = RMSEPopMat[,2],
			RmoveBias = biasPopMat[,4], RmoveRMSE = RMSEPopMat[,3])

##################################
# **** Table 8 for Paper: ****** #
##################################
round(popCondErrs,3)

######################################################
# ****  Figure 4 (right hand side) for Paper: ****** #
######################################################
par(mfrow = c(1,1), mar = c(4,4,1,1), cex.lab = 1.2, mgp = c(2.5,1,0))
colList = c("red", "darkblue", "darkgreen", "darkorange")
symList = c(1:4)
plot(thetaMuHat ~ RmoveHat, data = popParamDat, type = "n", ylim = c(-2.3, 2),
	ylab = expression(hat(mu)), xlab = expression(widehat(R)[move]))
points(thetaMuHat ~ RmoveHat, data = subset(popParamDat, ability == "H" & motive == "H"), 
	col = colList[1], pch = symList[1])
points(thetaMuHat ~ RmoveHat, data = subset(popParamDat, ability == "H" & motive == "L"), 
	col = colList[2], pch = symList[2])
points(thetaMuHat ~ RmoveHat, data = subset(popParamDat, ability == "L" & motive == "H"), 
	col = colList[3], pch = symList[3])
points(thetaMuHat ~ RmoveHat, data = subset(popParamDat, ability == "L" & motive == "L"), 
	col = colList[4], pch = symList[4])
lOrd = c(2,4,1,3)
legend(-1, -1.25, ncol = 2, col = colList[lOrd], pch = symList[lOrd], pt.lwd = 2, legend = popLabs[lOrd], 
	title = "Pop. Samples", bg = "white")

# Get the board stats:
boardFiles = paste("../boards/", boardList,".txt", sep = "")
for(b in 1:length(boardNames)) {
	sysCmd = paste("../PegBoardGameSim_ln.exe --boardStats --boardFile", boardFiles[b],
				"--discount", estDiscount, "--allowReset 0")
	system(sysCmd)
}

thetaErrMat = cbind(thetaErrMat, leaf_nodes = c(3, 11, 42, 47, 1454))
thetaErrMat = cbind(thetaErrMat, as_size = c(6, 12, 22, 30, 70))

##########################################################
# Looking at the actual scores for the plays

endReps = nReps
S4_playScoresList = vector(mode = "list")
S4_meanScores = matrix(nrow = 16, ncol = endReps )
S4_perfectPlay = vector(mode = "list")
S4_numReset = matrix(nrow = 16, ncol = endReps )
S4_dirList = vector()

setwd(simDir)
d = 1

for(b in 1:length(boardList)) {
for(c in 1:length(popLabs)) { 

	setwd(simDir)
	gendir = paste(boardList[b],popLabs[c],sep = "_")
	S4_dirList = c(S4_dirList, gendir)

	setwd(gendir)
	S4_playScoresList[[d]] = matrix(NA, nrow = endReps, ncol = N, dimnames = list(paste("run",runStr[1:endReps], sep="_"), paste("stud", studStr, sep="_")))
	S4_perfectPlay[[d]] = matrix(NA, nrow = endReps, ncol = N, dimnames = list(paste("run",runStr[1:endReps], sep="_"), paste("stud", studStr, sep="_")))
	
	for(run in 1:endReps) {
		playRecFile = paste("S4_playRecs_r", runStr[run], ".out", sep = "")
		if(!file.exists(playRecFile)) next
		playRecDat = read.table(playRecFile, fill = T)
		playRecScores = playRecDat[playRecDat[,2] == "Score",]
		resetDat = playRecDat[playRecDat[,2] == "Reset",]
		S4_numReset[d,run] = nrow(resetDat)
		reseters = unique(resetDat$V1)
		S4_playScoresList[[d]][run,] = as.numeric(as.character(playRecScores[,3]))
		S4_meanScores[d,run] = mean(S4_playScoresList[[d]][run,])
		S4_perfectPlay[[d]][run,] = as.numeric(!(1:N %in% reseters)) *  as.numeric(S4_playScoresList[[d]][run,] == 5)
	}
	setwd("..")
	d = d + 1
}
}
names(S4_playScoresList) = S4_dirList
names(S4_perfectPlay) = S4_dirList
rownames(S4_numReset) = S4_dirList

meanNumReset = apply(S4_numReset, 1, mean)
meanScore = apply(S4_meanScores, 1, mean)

# Aggregated by board and condition
resetDat = data.frame(popLabs = rep(popLabs, length(boardList)), 
	ab = rep(c("H","H","L","L"),  length(boardList)),
	mot = rep(c("H","L","H","L"), length(boardList)),
	 meanNumReset, meanScore)


####################################################
# ****  Table 7 (left hand side) for Paper: ****** #
####################################################
# Aggregated by condition
aggregate(meanNumReset ~ popLabs, data = resetDat, mean)
aggregate(meanScore ~ popLabs, data = resetDat, mean)

# Aggregated by run
playRecDat = data.frame(runId = c(1:(length(boardList)*length(popLabs)*endReps)) ) 
ind = 1
d = 1
for(b in 1:length(boardList)) {
for(c in 1:length(popLabs)) { 
for(run in 1:endReps) {
	playRecDat$board[ind] = boardList[b]
	playRecDat$pop[ind] = popLabs[c]
	if(c < 3)
		playRecDat$ab[ind] = "H"
	else
		playRecDat$ab[ind] = "L"
	if(c %in% c(1,3))
		playRecDat$mot[ind] = "H"
	else
		playRecDat$mot[ind] = "L"

	playRecDat$meanScore[ind] = mean(S4_playScoresList[[d]][run,])
	playRecDat$sdScore[ind] = sd(S4_playScoresList[[d]][run,])
	playRecDat$numPerfPlay[ind] = sum(S4_perfectPlay[[d]][run,])
	ind = ind + 1
}
d = d + 1
}
}

#######################################################
# **** Figure 4.  (left hand side) for Paper:  ****** #
#######################################################
boxplot(meanScore ~ pop, data = playRecDat, ylab = "Mean Total Score", xlab = "")


